@@ -1,48 +1,33 @@
 # Copyright (C) 2013, 2014 Red Hat, Inc.
 #
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-# MA 02110-1301 USA.
+# This work is licensed under the GNU GPLv2 or later.
+# See the COPYING file in the top-level directory.
 
 import atexit
 import imp
-import logging
 import os
 
 # Need to do this before any tests or virtinst import
 os.environ["VIRTINST_TEST_SUITE"] = "1"
-os.environ["VIRTINST_TEST_URL_DIR"] = os.path.abspath(
-    "tests/cli-test-xml/fakefedoratree/")
+# Need to do this before we import argcomplete
+os.environ.pop("_ARC_DEBUG", None)
 
 # pylint: disable=wrong-import-position
-from virtcli import cliconfig
+from virtinst import buildconfig
+from virtinst import log
 # This sets all the cli bits back to their defaults
-reload(cliconfig)
+imp.reload(buildconfig)
 
 from tests import utils
 
-# Variable used to store a local iso or dir path to check for a distro
-# Specified via 'python setup.py test_urls --path"
-URLTEST_LOCAL_MEDIA = []
-
 virtinstall = None
 virtclone = None
 virtconvert = None
 virtxml = None
 
 
-def _setup_logging():
+def setup_logging():
+    import logging
     rootLogger = logging.getLogger()
     for handler in rootLogger.handlers:
         rootLogger.removeHandler(handler)
@@ -50,13 +35,13 @@ def _setup_logging():
     logging.basicConfig(level=logging.DEBUG,
                         format="%(levelname)-8s %(message)s")
 
-    if utils.get_debug():
+    if utils.clistate.debug:
         rootLogger.setLevel(logging.DEBUG)
     else:
         rootLogger.setLevel(logging.ERROR)
 
 
-def _setup_cli_imports():
+def setup_cli_imports():
     _cleanup_imports = []
 
     def _cleanup_imports_cb():
@@ -77,7 +62,3 @@ def _setup_cli_imports():
     virtclone = _import("virtclone", "virt-clone")
     virtconvert = _import("virtconvert", "virt-convert")
     virtxml = _import("virtxml", "virt-xml")
-
-
-_setup_logging()
-_setup_cli_imports()
