@@ -1,25 +1,13 @@
 # Copyright (C) 2013 Red Hat, Inc.
 #
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-# MA 02110-1301 USA.
+# This work is licensed under the GNU GPLv2 or later.
+# See the COPYING file in the top-level directory.
 
-import logging
 import os
 import unittest
 
 from virtinst import StoragePool, StorageVolume
+from virtinst import log
 
 from tests import utils
 
@@ -39,26 +27,34 @@ def createPool(conn, ptype, poolname=None, fmt=None, target_path=None,
     pool_inst.name = poolname
     pool_inst.type = ptype
 
-    if pool_inst.supports_property("hosts"):
-        pool_inst.add_host("some.random.hostname")
-    if pool_inst.supports_property("source_path"):
+    if pool_inst.supports_hosts():
+        hostobj = pool_inst.hosts.add_new()
+        hostobj.name = "some.random.hostname"
+    if pool_inst.supports_source_path():
         pool_inst.source_path = source_path or "/some/source/path"
-    if pool_inst.supports_property("target_path"):
-        pool_inst.target_path = target_path or "/some/target/path"
-    if fmt and pool_inst.supports_property("format"):
+    if pool_inst.supports_target_path():
+        pool_inst.target_path = (target_path or
+                pool_inst.default_target_path())
+    if fmt and pool_inst.supports_format():
         pool_inst.format = fmt
-    if source_name and pool_inst.supports_property("source_name"):
-        pool_inst.source_name = source_name
-    if iqn and pool_inst.supports_property("iqn"):
+    if pool_inst.supports_source_name():
+        pool_inst.source_name = (source_name or
+                pool_inst.default_source_name())
+    if iqn and pool_inst.supports_iqn():
         pool_inst.iqn = iqn
 
-    pool_inst.validate()
     return poolCompare(pool_inst)
 
 
+def removePool(poolobj):
+    poolobj.destroy()
+    poolobj.undefine()
+
+
 def poolCompare(pool_inst):
+    pool_inst.validate()
     filename = os.path.join(basepath, pool_inst.name + ".xml")
-    out_expect = pool_inst.get_xml_config()
+    out_expect = pool_inst.get_xml()
 
     if not os.path.exists(filename):
         open(filename, "w").write(out_expect)
@@ -73,7 +69,7 @@ def createVol(conn, poolobj, volname=None, input_vol=None, clone_vol=None):
 
     # Format here depends on libvirt-1.2.0 and later
     if clone_vol and conn.local_libvirt_version() < 1002000:
-        logging.debug("skip clone compare")
+        log.debug("skip clone compare")
         return
 
     alloc = 5 * 1024 * 1024 * 1024
@@ -99,14 +95,14 @@ def createVol(conn, poolobj, volname=None, input_vol=None, clone_vol=None):
 
     vol_inst.validate()
     filename = os.path.join(basepath, vol_inst.name + ".xml")
-    utils.diff_compare(vol_inst.get_xml_config(), filename)
+    utils.diff_compare(vol_inst.get_xml(), filename)
     return vol_inst.install(meter=False)
 
 
 class TestStorage(unittest.TestCase):
-
-    def setUp(self):
-        self.conn = utils.open_testdefault()
+    @property
+    def conn(self):
+        return utils.URIs.open_testdefault_cached()
 
     def testDirPool(self):
         poolobj = createPool(self.conn,
@@ -116,6 +112,7 @@ class TestStorage(unittest.TestCase):
                   volname=invol.name() + "input", input_vol=invol)
         createVol(self.conn, poolobj,
                   volname=invol.name() + "clone", clone_vol=invol)
+        removePool(poolobj)
 
     def testFSPool(self):
         poolobj = createPool(self.conn,
@@ -125,6 +122,7 @@ class TestStorage(unittest.TestCase):
                   volname=invol.name() + "input", input_vol=invol)
         createVol(self.conn, poolobj,
                   volname=invol.name() + "clone", clone_vol=invol)
+        removePool(poolobj)
 
     def testNetFSPool(self):
         poolobj = createPool(self.conn,
@@ -134,93 +132,94 @@ class TestStorage(unittest.TestCase):
                   volname=invol.name() + "input", input_vol=invol)
         createVol(self.conn, poolobj,
                   volname=invol.name() + "clone", clone_vol=invol)
+        removePool(poolobj)
 
     def testLVPool(self):
         poolobj = createPool(self.conn,
                              StoragePool.TYPE_LOGICAL,
                              "pool-logical",
-                             target_path="/dev/pool-logical")
+                             source_name="pool-logical")
         invol = createVol(self.conn, poolobj)
         createVol(self.conn, poolobj,
                   volname=invol.name() + "input", input_vol=invol)
         createVol(self.conn,
                   poolobj, volname=invol.name() + "clone", clone_vol=invol)
+        removePool(poolobj)
 
         # Test parsing source name for target path
-        createPool(self.conn, StoragePool.TYPE_LOGICAL,
+        poolobj = createPool(self.conn, StoragePool.TYPE_LOGICAL,
                    "pool-logical-target-srcname",
                    target_path="/dev/vgfoobar")
+        removePool(poolobj)
 
         # Test with source name
-        createPool(self.conn,
+        poolobj = createPool(self.conn,
                    StoragePool.TYPE_LOGICAL, "pool-logical-srcname",
                    source_name="vgname")
-
-        # Test creating with many devices
-        # XXX: Need to wire this up
-        # createPool(self.conn,
-        #            StoragePool.TYPE_LOGICAL, "pool-logical-manydev",
-        #            source_path=["/tmp/path1", "/tmp/path2", "/tmp/path3"],
-        #            target_path=None)
+        removePool(poolobj)
 
     def testDiskPool(self):
         poolobj = createPool(self.conn,
                              StoragePool.TYPE_DISK,
-                             "pool-disk", fmt="dos")
+                             "pool-disk", fmt="auto",
+                             target_path="/some/target/path")
         invol = createVol(self.conn, poolobj)
         createVol(self.conn, poolobj,
                   volname=invol.name() + "input", input_vol=invol)
         createVol(self.conn, poolobj,
                   volname=invol.name() + "clone", clone_vol=invol)
+        removePool(poolobj)
 
     def testISCSIPool(self):
-        createPool(self.conn,
+        poolobj = createPool(self.conn,
                    StoragePool.TYPE_ISCSI, "pool-iscsi",
                    iqn="foo.bar.baz.iqn")
+        removePool(poolobj)
 
     def testSCSIPool(self):
-        createPool(self.conn, StoragePool.TYPE_SCSI, "pool-scsi")
+        poolobj = createPool(self.conn, StoragePool.TYPE_SCSI, "pool-scsi")
+        removePool(poolobj)
 
     def testMpathPool(self):
-        createPool(self.conn, StoragePool.TYPE_MPATH, "pool-mpath")
+        poolobj = createPool(self.conn, StoragePool.TYPE_MPATH, "pool-mpath")
+        removePool(poolobj)
 
     def testGlusterPool(self):
-        if not self.conn.check_support(self.conn.SUPPORT_CONN_POOL_GLUSTERFS):
-            raise unittest.SkipTest("Gluster pools not supported with this "
-                "libvirt version.")
+        poolobj = createPool(self.conn,
+                StoragePool.TYPE_GLUSTER, "pool-gluster")
+        removePool(poolobj)
 
-        createPool(self.conn, StoragePool.TYPE_GLUSTER, "pool-gluster")
+    def testRBDPool(self):
+        poolobj = createPool(self.conn,
+                StoragePool.TYPE_RBD, "pool-rbd")
+        removePool(poolobj)
 
+    def testMisc(self):
+        # Misc coverage testing
+        vol = StorageVolume(self.conn)
+        self.assertTrue(vol.is_size_conflict()[0] is False)
 
-    ##############################
-    # Tests for pool-sources API #
-    ##############################
+        fullconn = utils.URIs.open_testdriver_cached()
+        glusterpool = fullconn.storagePoolLookupByName("gluster-pool")
+        diskpool = fullconn.storagePoolLookupByName("disk-pool")
 
-    def _enumerateCompare(self, name, pool_list):
-        for pool in pool_list:
-            pool.name = name + str(pool_list.index(pool))
-            poolCompare(pool)
+        glustervol = StorageVolume(fullconn)
+        glustervol.pool = glusterpool
+        self.assertTrue(glustervol.supports_format() is True)
+
+        diskvol = StorageVolume(fullconn)
+        diskvol.pool = diskpool
+        self.assertTrue(diskvol.supports_format() is False)
+
+        glusterpool.destroy()
+        StoragePool.ensure_pool_is_running(glusterpool)
+
+        # Check pool collision detection
+        self.assertEqual(
+                StoragePool.find_free_name(fullconn, "gluster-pool"),
+                "gluster-pool-1")
 
     def testEnumerateLogical(self):
-        name = "pool-logical-list"
         lst = StoragePool.pool_list_from_sources(self.conn,
                                                  StoragePool.TYPE_LOGICAL)
-        self._enumerateCompare(name, lst)
-
-    def testEnumerateNetFS(self):
-        name = "pool-netfs-list"
-        host = "example.com"
-        lst = StoragePool.pool_list_from_sources(self.conn,
-                                                 StoragePool.TYPE_NETFS,
-                                                 host=host)
-        self._enumerateCompare(name, lst)
-
-    def testEnumerateiSCSI(self):
-        host = "example.com"
-        lst = StoragePool.pool_list_from_sources(self.conn,
-                                                 StoragePool.TYPE_ISCSI,
-                                                 host=host)
-        self.assertTrue(len(lst) == 0)
-
-if __name__ == "__main__":
-    unittest.main()
+        self.assertEqual(lst, ["testvg1", "testvg2"])
