@@ -1,6 +1,6 @@
-from __future__ import print_function
+# This work is licensed under the GNU GPLv2 or later.
+# See the COPYING file in the top-level directory.
 
-import logging
 import os
 import re
 import time
@@ -9,18 +9,29 @@ import subprocess
 import sys
 import unittest
 
+from gi.repository import Gio
+from gi.repository import Gdk
 import pyatspi
-import dogtail.tree
+import dogtail.utils
+
+from virtinst import log
 
 import tests
 
+if not dogtail.utils.isA11yEnabled():
+    print("Enabling gsettings accessibility")
+    dogtail.utils.enableA11y()
+
+# This will trigger an error if accessibility isn't enabled
+import dogtail.tree  # pylint: disable=wrong-import-order,ungrouped-imports
+
 
 class UITestCase(unittest.TestCase):
     """
     Common testcase bits shared for ui tests
     """
     def setUp(self):
-        self.app = VMMDogtailApp(tests.utils.uri_test)
+        self.app = VMMDogtailApp(tests.utils.URIs.test_full)
     def tearDown(self):
         self.app.stop()
 
@@ -67,11 +78,16 @@ class UITestCase(unittest.TestCase):
         win.find_fuzzy(tab, "page tab").click()
         return win
 
-    def _open_details_window(self, vmname=None, shutdown=False):
+    def _open_details_window(self, vmname=None, shutdown=False,
+            double=False):
         if vmname is None:
             vmname = self._default_vmname
-        self.app.root.find_fuzzy(vmname, "table cell").click(button=3)
-        self.app.root.find("Open", "menu item").click()
+
+        if double:
+            self.app.root.find_fuzzy(vmname, "table cell").doubleClick()
+        else:
+            self.app.root.find_fuzzy(vmname, "table cell").click(button=3)
+            self.app.root.find("Open", "menu item").click()
 
         win = self.app.root.find("%s on" % vmname, "frame")
         win.find("Details", "radio button").click()
@@ -81,7 +97,7 @@ class UITestCase(unittest.TestCase):
             check_in_loop(lambda: run.sensitive)
         return win
 
-    def _walkUIList(self, win, lst, error_cb):
+    def _walkUIList(self, win, lst, error_cb, reverse=False):
         """
         Toggle down through a UI list like addhardware, net/storage/iface
         lists, and ensure an error isn't raised.
@@ -89,6 +105,8 @@ class UITestCase(unittest.TestCase):
         # Walk the lst UI and find all labelled table cells, these are
         # the actual list entries
         all_cells = lst.findChildren(lambda w: w.roleName == "table cell")
+        if reverse:
+            all_cells.reverse()
         all_cells[0].click()
         cells_per_selection = len([c for c in all_cells if c.focused])
 
@@ -103,7 +121,7 @@ class UITestCase(unittest.TestCase):
                     continue
 
             self.assertTrue(cell.state_selected)
-            dogtail.rawinput.pressKey("Down")
+            dogtail.rawinput.pressKey(reverse and "Up" or "Down")
 
             if not win.active:
                 # Should mean an error dialog popped up
@@ -119,6 +137,36 @@ class UITestCase(unittest.TestCase):
             else:
                 self.assertTrue(not cell.state_selected)
 
+    def _test_xmleditor_interactions(self, win, finish):
+        """
+        Helper to test some common XML editor interactions
+        """
+        # Click the tab, make a bogus XML edit
+        win.find("XML", "page tab").click()
+        xmleditor = win.find("XML editor")
+        xmleditor.text = xmleditor.text.replace("<", "<FOO", 1)
+
+        # Trying to click away should warn that there's unapplied changes
+        win.find("Details", "page tab").click()
+        alert = self.app.root.find("vmm dialog")
+        alert.find_fuzzy("changes will be lost")
+
+        # Select 'No', meaning don't abandon changes
+        alert.find("No", "push button").click()
+        check_in_loop(lambda: xmleditor.showing)
+
+        # Click the finish button, but our bogus change should trigger error
+        finish.click()
+        alert = self.app.root.find("vmm dialog")
+        alert.find_fuzzy("(xmlParseDoc|tag mismatch)")
+        alert.find("Close", "push button").click()
+
+        # Try unapplied changes again, this time abandon our changes
+        win.find("Details", "page tab").click()
+        alert = self.app.root.find("vmm dialog")
+        alert.find("Yes", "push button").click()
+        check_in_loop(lambda: not xmleditor.showing)
+
 
 class _FuzzyPredicate(dogtail.predicate.Predicate):
     """
@@ -176,7 +224,9 @@ class _FuzzyPredicate(dogtail.predicate.Predicate):
                 return
             return True
         except Exception as e:
-            print("got predicate exception: %s" % e)
+            log.debug(
+                    "got predicate exception name=%s role=%s labeller=%s: %s",
+                    self._name, self._roleName, self._labeller_text, e)
 
 
 def check_in_loop(func, timeout=2):
@@ -194,6 +244,16 @@ def check_in_loop(func, timeout=2):
         time.sleep(interval)
 
 
+def drag(win, x, y):
+    """
+    Drag a window to the x/y coordinates
+    """
+    win.click()
+    clickX = win.position[0] + win.size[0] / 2
+    clickY = win.position[1] + 10
+    dogtail.rawinput.drag((clickX, clickY), (x, y))
+
+
 class VMMDogtailNode(dogtail.tree.Node):
     """
     Our extensions to the dogtail node wrapper class.
@@ -214,15 +274,36 @@ class VMMDogtailNode(dogtail.tree.Node):
 
     @property
     def onscreen(self):
-        return self.position[0] > 0 and self.position[1] > 0
+        # We need to check that full widget is on screen because we use this
+        # function to check whether we can click a widget. We may click
+        # anywhere within the widget and clicks outside the screen bounds are
+        # silently ignored.
+        if self.roleName in ["menu", "menu item", "frame"]:
+            return True
+        screen = Gdk.Screen.get_default()
+        return (self.position[0] > 0 and
+                self.position[0] + self.size[0] < screen.get_width() and
+                self.position[1] > 0 and
+                self.position[1] + self.size[1] < screen.get_height())
+
+    def click_secondary_icon(self):
+        """
+        Helper for clicking the secondary icon of a text entry
+        """
+        button = 1
+        clickX = self.position[0] + self.size[0] - 10
+        clickY = self.position[1] + (self.size[1] / 2)
+        dogtail.rawinput.click(clickX, clickY, button)
 
     def click_combo_entry(self):
         """
         Helper for clicking the arrow of a combo entry, to expose the menu.
-        Clicks middle of Y axis, but 1 pixel in from the right side
+        Clicks middle of Y axis, but 1/10th of the height from the right side.
+        Using a small, hardcoded offset may not work on some themes (e.g. when
+        running virt-manager on KDE)
         """
         button = 1
-        clickX = self.position[0] + self.size[0] - 1
+        clickX = self.position[0] + self.size[0] - self.size[1] / 4
         clickY = self.position[1] + self.size[1] / 2
         dogtail.rawinput.click(clickX, clickY, button)
 
@@ -236,12 +317,34 @@ class VMMDogtailNode(dogtail.tree.Node):
         clickY = self.position[1] + 5
         dogtail.rawinput.click(clickX, clickY, button)
 
+    def click(self, *args, **kwargs):
+        """
+        click wrapper, give up to a second for widget to appear on
+        screen, helps reduce some test flakiness
+        """
+        # pylint: disable=arguments-differ
+        check_in_loop(lambda: self.onscreen)
+        dogtail.tree.Node.click(self, *args, **kwargs)
+
+    def bring_on_screen(self, key_name="Down", max_tries=100):
+        """
+        Attempts to bring the item to screen by repeatedly clicking the given
+        key. Raises exception if max_tries attempts are exceeded.
+        """
+        cur_try = 0
+        while not self.onscreen:
+            dogtail.rawinput.pressKey(key_name)
+            cur_try += 1
+            if cur_try > max_tries:
+                raise RuntimeError("Could not bring widget on screen")
+        return self
+
 
     #########################
     # Widget search helpers #
     #########################
 
-    def find(self, name, roleName=None, labeller_text=None):
+    def find(self, name, roleName=None, labeller_text=None, check_active=True):
         """
         Search root for any widget that contains the passed name/role regex
         strings.
@@ -258,7 +361,7 @@ class VMMDogtailNode(dogtail.tree.Node):
         # Wait for independent windows to become active in the window manager
         # before we return them. This ensures the window is actually onscreen
         # so it sidesteps a lot of race conditions
-        if ret.roleName in ["frame", "dialog", "alert"]:
+        if ret.roleName in ["frame", "dialog", "alert"] and check_active:
             check_in_loop(lambda: ret.active)
         return ret
 
@@ -289,18 +392,23 @@ class VMMDogtailNode(dogtail.tree.Node):
             msg += " labeller.text='%s'" % self.labeller.text
         return msg
 
+    def fmt_nodes(self):
+        strs = []
+        def _walk(node):
+            try:
+                strs.append(node.node_string())
+            except Exception as e:
+                strs.append("got exception: %s" % e)
+
+        self.findChildren(_walk, isLambda=True)
+        return "\n".join(strs)
+
     def print_nodes(self):
         """
         Helper to print the entire node tree for the passed root. Useful
         if to figure out the roleName for the object you are looking for
         """
-        def _walk(node):
-            try:
-                print(node.node_string())
-            except Exception as e:
-                print("got exception: %s" % e)
-
-        self.findChildren(_walk, isLambda=True)
+        print(self.fmt_nodes())
 
 
 # This is the same hack dogtail uses to extend the Accessible class.
@@ -333,13 +441,24 @@ class VMMDogtailApp(object):
             self.open()
         return self._topwin
 
+    def error_if_already_running(self):
+        # Ensure virt-manager isn't already running
+        dbus = Gio.DBusProxy.new_sync(
+                Gio.bus_get_sync(Gio.BusType.SESSION, None), 0, None,
+                "org.freedesktop.DBus", "/org/freedesktop/DBus",
+                "org.freedesktop.DBus", None)
+        if "org.virt-manager.virt-manager" in dbus.ListNames():
+            raise RuntimeError("virt-manager is already running. "
+                    "Close it before running this test suite.")
+
     def is_running(self):
         return bool(self._proc and self._proc.poll() is None)
 
-    def open(self, extra_opts=None):
+    def open(self, extra_opts=None, check_already_running=True, use_uri=True,
+            window_name=None, xmleditor_enabled=False):
         extra_opts = extra_opts or []
 
-        if tests.utils.get_debug():
+        if tests.utils.clistate.debug:
             stdout = sys.stdout
             stderr = sys.stderr
             extra_opts.append("--debug")
@@ -349,14 +468,23 @@ class VMMDogtailApp(object):
 
         cmd = [sys.executable]
         if tests.utils.clistate.use_coverage:
-            cmd += ["-m", "coverage", "run", "--append"]
+            cmd += ["-m", "coverage", "run", "--append",
+                    "--omit", "/usr/*",
+                    "--omit", "*/virtinst/*"]
         cmd += [os.path.join(os.getcwd(), "virt-manager"),
-                "--test-first-run", "--no-fork", "--connect", self.uri]
+                "--test-first-run",
+                "--no-fork"]
+        if use_uri:
+            cmd += ["--connect", self.uri]
+        if xmleditor_enabled:
+            cmd += ["--test-options=xmleditor-enabled"]
         cmd += extra_opts
 
+        if check_already_running:
+            self.error_if_already_running()
         self._proc = subprocess.Popen(cmd, stdout=stdout, stderr=stderr)
         self._root = dogtail.tree.root.application("virt-manager")
-        self._topwin = self._root.find(None, "(frame|dialog|alert)")
+        self._topwin = self._root.find(window_name, "(frame|dialog|alert)")
 
     def stop(self):
         """
@@ -368,7 +496,7 @@ class VMMDogtailApp(object):
         try:
             self._proc.send_signal(signal.SIGINT)
         except Exception:
-            logging.debug("Error terminating process", exc_info=True)
+            log.debug("Error terminating process", exc_info=True)
             self._proc = None
             return
 
@@ -379,7 +507,7 @@ class VMMDogtailApp(object):
                 self._proc = None
                 return
 
-        logging.warning("App didn't exit gracefully from SIGINT. Killing...")
+        log.warning("App didn't exit gracefully from SIGINT. Killing...")
         try:
             self._proc.kill()
         finally:
