@@ -3,30 +3,17 @@
 # Copyright 2008 Sun Microsystems, Inc.  All rights reserved.
 # Use is subject to license terms.
 #
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-# MA 02110-1301 USA.
+# This work is licensed under the GNU GPLv2 or later.
+# See the COPYING file in the top-level directory.
 #
 
-from distutils.spawn import find_executable
-import logging
 import os
 import re
 import shutil
 import subprocess
 import tempfile
 
+from virtinst import log
 from virtinst import StoragePool
 
 
@@ -87,15 +74,15 @@ def _run_cmd(cmd):
     """
     Return the exit status and output to stdout and stderr.
     """
-    logging.debug("Running command: %s", " ".join(cmd))
+    log.debug("Running command: %s", " ".join(cmd))
     proc = subprocess.Popen(cmd, stderr=subprocess.PIPE,
                             stdout=subprocess.PIPE,
                             close_fds=True)
     stdout, stderr = proc.communicate()
     ret = proc.wait()
 
-    logging.debug("stdout=%s", stdout)
-    logging.debug("stderr=%s", stderr)
+    log.debug("stdout=%s", stdout)
+    log.debug("stderr=%s", stderr)
 
     if ret == 0:
         return
@@ -118,6 +105,8 @@ def _find_input(input_file, parser, print_cb):
     try:
         ext = os.path.splitext(input_file)[1]
         tempdir = None
+        binname = None
+        pkg = None
         if ext and ext[1:] in ["zip", "gz", "ova",
                 "tar", "bz2", "bzip2", "7z", "xz"]:
             basedir = "/var/tmp"
@@ -129,19 +118,40 @@ def _find_input(input_file, parser, print_cb):
 
             base = os.path.basename(input_file)
 
-            # check if 'unar' command existed.
-            if not find_executable("unar"):
+            if (ext[1:] == "zip"):
+                binname = "unzip"
+                pkg = "unzip"
+                cmd = ["unzip", "-o", "-d", tempdir, input_file]
+            elif (ext[1:] == "7z"):
+                binname = "7z"
+                pkg = "p7zip"
+                cmd = ["7z", "-o" + tempdir, "e", input_file]
+            elif (ext[1:] == "ova" or ext[1:] == "tar"):
+                binname = "tar"
+                pkg = "tar"
+                cmd = ["tar", "xf", input_file, "-C", tempdir]
+            elif (ext[1:] == "gz"):
+                binname = "gzip"
+                pkg = "gzip"
+                cmd = ["tar", "zxf", input_file, "-C", tempdir]
+            elif (ext[1:] == "bz2" or ext[1:] == "bzip2"):
+                binname = "bzip2"
+                pkg = "bzip2"
+                cmd = ["tar", "jxf", input_file, "-C", tempdir]
+            elif (ext[1:] == "xz"):
+                binname = "xz"
+                pkg = "xz"
+                cmd = ["tar", "Jxf", input_file, "-C", tempdir]
+            if not shutil.which(binname):
                 raise RuntimeError(_("%s appears to be an archive, "
-                    "but 'unar' is not installed. "
-                    "Please either install 'unar', or extract the archive "
+                    "but '%s' is not installed. "
+                    "Please either install '%s', or extract the archive "
                     "yourself and point virt-convert at "
-                    "the extracted directory.") % base)
+                    "the extracted directory.") % (base, pkg, pkg))
 
-            cmd = ["unar", "-o", tempdir, base]
             print_cb(_("%s appears to be an archive, running: %s") %
                 (base, " ".join(cmd)))
 
-            cmd[-1] = input_file
             _run_cmd(cmd)
             force_clean.append(tempdir)
             input_file = tempdir
@@ -160,7 +170,7 @@ def _find_input(input_file, parser, print_cb):
                         return path, p, force_clean
 
         raise RuntimeError("Could not find parser for file %s" % input_file)
-    except:
+    except Exception:
         for f in force_clean:
             shutil.rmtree(f)
         raise
@@ -175,10 +185,11 @@ class VirtConverter(object):
         self._err_clean = []
         self._force_clean = []
 
+        # pylint: disable=redefined-variable-type
         if print_cb == -1 or print_cb is None:
             def cb(msg):
                 if print_cb == -1:
-                    print msg
+                    print(msg)
             self.print_cb = cb
         else:
             self.print_cb = print_cb
@@ -188,7 +199,7 @@ class VirtConverter(object):
             parser = _find_parser_by_name(input_name)
 
         input_file = os.path.abspath(input_file)
-        logging.debug("converter __init__ with input=%s parser=%s",
+        log.debug("converter __init__ with input=%s parser=%s",
             input_file, parser)
 
         (self._input_file,
@@ -196,17 +207,11 @@ class VirtConverter(object):
          self._force_clean) = _find_input(input_file, parser, self.print_cb)
         self._top_dir = os.path.dirname(os.path.abspath(self._input_file))
 
-        logging.debug("converter not input_file=%s parser=%s",
+        log.debug("converter not input_file=%s parser=%s",
             self._input_file, self.parser)
 
-        cwd = os.getcwd()
-        try:
-            os.chdir(self._top_dir)
-            self._guest = self.parser.export_libvirt(self.conn,
-                self._input_file)
-            self._guest.add_default_devices()
-        finally:
-            os.chdir(cwd)
+        self._guest = self.parser.export_libvirt(self.conn,
+            self._input_file)
 
     def __del__(self):
         for f in self._force_clean:
@@ -240,11 +245,13 @@ class VirtConverter(object):
         """
         binnames = ["qemu-img", "kvm-img"]
 
+        decompress_cmd = None
+
         if _is_test():
             executable = "/usr/bin/qemu-img"
         else:
             for binname in binnames:
-                executable = find_executable(binname)
+                executable = shutil.which(binname)
                 if executable:
                     break
 
@@ -252,12 +259,23 @@ class VirtConverter(object):
             raise RuntimeError(_("None of %s tools found.") % binnames)
 
         base = os.path.basename(absin)
+        ext = os.path.splitext(base)[1]
+        if (ext and ext[1:] == "gz"):
+            if not shutil.which("gzip"):
+                raise RuntimeError("'gzip' is needed to decompress the file, "
+                    "but not found.")
+            decompress_cmd = ["gzip", "-d", absin]
+            base = os.path.splitext(base)[0]
+            absin = absin[0:-3]
+            self.print_cb("Running %s" % " ".join(decompress_cmd))
         cmd = [executable, "convert", "-O", disk_format, base, absout]
         self.print_cb("Running %s" % " ".join(cmd))
         if dry:
             return
 
         cmd[4] = absin
+        if decompress_cmd is not None:
+            _run_cmd(decompress_cmd)
         _run_cmd(cmd)
 
     def convert_disks(self, disk_format, destdir=None, dry=False):
@@ -269,15 +287,16 @@ class VirtConverter(object):
             disk_format = None
 
         if destdir is None:
-            destdir = StoragePool.get_default_dir(self.conn, build=not dry)
+            poolxml = StoragePool.build_default_pool(self.conn)
+            destdir = poolxml.target_path
 
         guest = self.get_guest()
-        for disk in guest.get_devices("disk"):
+        for disk in guest.devices.disk:
             if disk.device != "disk":
                 continue
 
             if disk_format and disk.driver_type == disk_format:
-                logging.debug("path=%s is already in requested format=%s",
+                log.debug("path=%s is already in requested format=%s",
                     disk.path, disk_format)
                 disk_format = None
 
@@ -286,7 +305,7 @@ class VirtConverter(object):
             if disk_format:
                 newpath += ("." + disk_format)
             newpath = os.path.join(destdir, newpath)
-            if os.path.exists(newpath):
+            if os.path.exists(newpath) and not _is_test():
                 raise RuntimeError(_("New path name '%s' already exists") %
                     newpath)
 
