@@ -13,6 +13,8 @@ class DeviceMemballoon(Device):
     XML_NAME = "memballoon"
 
     model = XMLProperty("./@model")
+    autodeflate = XMLProperty("./@autodeflate", is_onoff=True)
+    stats_period = XMLProperty("./stats/@period", is_int=True)
 
 
     ##################
